#include <stdio.h>
#include <limits.h>

#include "JavaDVM.h"

#define	_debug	(1)
#define getJavaString(j)        (*env)->GetStringUTFChars(env,(j), 0)
#define releaseJavaString(j,c)  (*env)->ReleaseStringUTFChars(env,(j),(c))


/*
 * JavaDVM.vdcemain()
 */

FILE *debugfilep;

JNIEXPORT jint JNICALL Java_JavaDVM_vdcemain (
          JNIEnv        *env, 
          jobject	obj, 
          jobjectArray  jArgs, 
          jobjectArray  jTaskArgs)
{
    int 	i;
    char 	*cTaskArgs[512];
    char 	*cArgs[512];
    int         cNumArgs;
    int         cNumTaskArgs;
    char	*fname;
    
    jint	result;
    jstring     jTaskArgv;

    jstring     *jArgvElements;
    jsize	jNumArgs;
    FILE	*fp;

    fname = (char *) calloc(128,sizeof(char));
    jNumArgs = (*env)->GetArrayLength(env,jArgs);
    jArgvElements = (jstring *) calloc(jNumArgs,sizeof(jstring));
   
    for (i=0;i<jNumArgs;i++) {
        cArgs[i] = (char *) calloc(128,sizeof(char)); 
        jArgvElements[i] = (jstring) 
		       (*env)->GetObjectArrayElement(env,jArgs,i);
        cArgs[i] = (char *) getJavaString(jArgvElements[i]);
    }
    cArgs[i] = NULL; 
    cNumArgs    = jNumArgs;



    sprintf(fname,"/tmp/%s%d",cArgs[0],atoi(cArgs[6]));
    fp = fopen(fname,"w+");
    if (_debug)
       fprintf(fp,"JavaDVM.vdcemain(%d,*,*,*)\n",cNumArgs-1);

    fflush(fp);
    /* call DVM vdce_main() */
    result = (jint) vdce_main(cNumArgs,cArgs,&cNumTaskArgs,cTaskArgs);

    /* if (result == 1) return 0;  error in main */ 
       	
    fprintf(fp,"Returned from vdce_main() without error\n");
    fflush(fp);
    /* Copy result to the jTaskArgs */
    for(i=0;i<cNumTaskArgs;i++) {
	/* get an element from C */	
	jTaskArgv = (*env)->NewStringUTF(env,cTaskArgs[i]);
	fprintf(fp,"cTaskArgs[%d]=%s\n",i,cTaskArgs[i]);
        fflush(fp);
	/* copy to Java */
	(*env)->SetObjectArrayElement(env,jArgs,i,jTaskArgv);
    }

    fflush(fp);
    debugfilep = fp;
    return (jint) cNumTaskArgs;
}

/*
 * 
 * JavaDVM.vdcerun()
 * 
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdcerun (
        JNIEnv		*env, 
        jobject		obj,
        jint		jArgc, 
        jobjectArray	jArgv)
{
	int		cArgc;
	char		**cArgv;

	int		i;
	jstring		*jArgvElements;
	jint		result_of_run;
	jsize		jNumArgs;

	/* allocate space for C arrays and copy Argv into it */
	jNumArgs = (*env)->GetArrayLength(env,jArgv);
	cArgv = (char **) calloc(jNumArgs+1,sizeof(char *));
        jArgvElements = (jstring *) calloc(jNumArgs,sizeof(jstring));

	for (i=0;i<jNumArgs;i++) {
	    jArgvElements[i] = (jstring) 
			       (*env)->GetObjectArrayElement(env,jArgv,i);
	    cArgv[i] = (char *) getJavaString(jArgvElements[i]);
	}
	cArgv[i] = NULL; 
	cArgc    = jNumArgs;

        /* call the DVM routine: vdce_run */
        result_of_run = (jint) vdce_run(cArgc,(char **)cArgv);	

	/* Release allocated memory */
	for (i=0;i<jNumArgs;i++)  
	    releaseJavaString(jArgvElements[i], cArgv[i]);

	free(jArgvElements);
	free(cArgv);

        fflush(debugfilep);
	return result_of_run;
}

/*
 * 
 * JavaDVM.vdceend()
 * 
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdceend(
	JNIEnv 		*env, 
        jobject		obj,
	jint		jFlag)
{

	if (_debug)
	    fprintf(debugfilep,"JavDVM.vdceend(%d)\n",jFlag);
		
        fflush(debugfilep);
	fclose(debugfilep);
	/* call DVM vdce_end() */
	return (jint) vdce_end((int) jFlag);
}
/*
 * 
 * JavaDVM.vdcestop()
 * 
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdcestop (
	JNIEnv 		*env, 
	jobject		obj,
	jint		jFlag)
{

	if (_debug)
	   fprintf(debugfilep,"JavaDVM.vdcestop(%d)\n",jFlag);

	/* call DVM vdce_stop() */
	return (jint) vdce_stop((int) jFlag);
}

/*
 * 
 * JavaDVM.vdceerror()
 * 
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdceerror(
  	JNIEnv 		*env,
	jobject		obj, 
	jint		jErrorcode)
{

	if (_debug)
	   fprintf(debugfilep,"JavaDVM.vdcerror(%d)\n",jErrorcode);

        fflush(debugfilep);
	/* call DVM vdce_error() */ 
	return (jint) vdce_error((int) jErrorcode);
}

/*
 * 
 * JavaDVM.vdcerecv()
 * 
 */
JNIEXPORT jstring JNICALL Java_JavaDVM_vdcerecv(
  	JNIEnv 		*env, 
 	jobject		obj,
	jint		jVdcePort)
{
	char		*cRecvBuf;
	int		i;
	int		nsize;

	jint		jBufLen;
	jstring   	jRxBuf;


	if (_debug) 
	   fprintf(debugfilep,"JavaDVM.vdcerecv(%d)\n",jVdcePort);

        fflush(debugfilep);
	jBufLen = (jint) vdce_recv((int) jVdcePort, &cRecvBuf);
        fprintf(debugfilep,"Data received = %d\n",(int) jBufLen);
        if (jBufLen < 1) return NULL; /* Data has not been arrived  */

	/* store char arrary at Java String buffer */
        jRxBuf = (*env)->NewStringUTF(env,cRecvBuf);
	
	if (_debug)
	  fprintf(debugfilep,"JavaDVM.vdcerecev() got %d amount of data\n",jBufLen);

        fflush(debugfilep);
	free(cRecvBuf);
	return jRxBuf;
}

/*
 * JavaDVM.vdcesend()
 * 
 * 
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdcesend(
   	JNIEnv 		*env, 
	jobject		obj,
	jint		jVdcePort, 
	jcharArray	jCharArray,
	jint		jNsize)
{
	char	*cTxBuf;
	int	i;

	jchar	*jTxBuf;
	jint	nsize;

	if (_debug)
	   fprintf(debugfilep,"JavaDVM.vdcesend(%d,*,%d)\n",jVdcePort,jNsize);

        fflush(debugfilep);
	/* get pointer to buf of the Java Array */
	jTxBuf = (*env)->GetCharArrayElements(env,jCharArray,0);
	cTxBuf = (char *) calloc((int)jNsize+1,sizeof(char));

	for(i=0;i<jNsize;i++) cTxBuf[i] = jTxBuf[i];
	cTxBuf[i] = '\0';
	/* call DVM vdce_send() */
	return (jint) vdce_send((int)jVdcePort,cTxBuf,(int)jNsize);
}

/*
 * 
 * JavaDVM.vdcegetport()
 * 
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdcegetport (
	JNIEnv 		*env, 
	jobject		obj,
	jint		jVdcePort, 
	jint		jTag)
{

        if (_debug)
	   fprintf(debugfilep,"JavaDVM.vdcegetport(%d,%d)\n",jVdcePort,jTag);	

        fflush(debugfilep);
	return (jint) vdce_port((int)jVdcePort,(int) jTag);
}

/*
 * 
 * JavaDVM.vdcedatarepresent()
 * 
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdcedatarepresent(
	JNIEnv 		*env, 
	jobject		obj,
	jint		jVdcePort, 
	jint		jTag)
{
        if (_debug)
	   fprintf(debugfilep,"JavaDVM.vdcedatarepresent(%d,%d)\n",jVdcePort,jTag);	

        fflush(debugfilep);
	return (jint) vdce_data_represent((int)jVdcePort,(int) jTag);
}

/*
 * Class:     JavaDVM
 * Method:    vdceinputname
 * Signature: (II)I
 */
JNIEXPORT jstring JNICALL Java_JavaDVM_vdceinputname(
  	JNIEnv 		*env, 
	jobject		obj,
	jint		jTaskid, 
	jint		jPortid)
{
	char		*cret;
	jstring		ret;

	if (_debug)
	   fprintf(debugfilep,"JavaDVM.vdceinputname(%d,%d)\n",jTaskid,jPortid);

        fflush(debugfilep);
	cret = (char *) calloc(1024,sizeof(char));
	cret=  (char *) vdce_input_name((int)jTaskid,(int) jPortid);
	
	if (_debug) 
		fprintf(debugfilep,"inputname=%s\n",cret);
        fflush(debugfilep);
	ret = (*env)->NewStringUTF(env,cret);
	free(cret);

	return ret;
}

/*
 *
 * JavaDVM.vdceoutputname()
 * 
 */
JNIEXPORT jstring JNICALL Java_JavaDVM_vdceoutputname(
  	JNIEnv 		*env, 
	jobject		obj,
	jint		jTaskid, 
	jint		jPortid)
{

	char		*cret;
	jstring		ret;

	if (_debug)
	   fprintf(debugfilep,"JavaDVM.vdceoutputname(%d,%d)\n",jTaskid,jPortid);
        fflush(debugfilep);

        cret = (char *) calloc(1024,sizeof(char));
        cret=  (char *) vdce_output_name((int)jTaskid,(int) jPortid);
        
        if (_debug) 
                fprintf(debugfilep,"inputname=%s\n",cret);
        ret = (*env)->NewStringUTF(env,cret);
        free(cret);
 
        return ret;
}

/*
 * 
 * JavaDVM.vdcegettaskinfo()
 * 
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdcegettaskinfo(
  	JNIEnv 		*env,
	jobject	obj)
{
	if (_debug)
	   fprintf(debugfilep,"JavaDVM.vdcegettaskinfo()\n");

        fflush(debugfilep);
	return (jint) vdce_get_taskinfo();
}


/*
 * 
 * JavaDVM.vdcegetmyid()
 * 
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdcegetmyid(
  	JNIEnv 		*env,
	jobject		obj)
{

 	if (_debug)
	    fprintf(debugfilep,"JavaDVM.vdcegetmyid()\n");
	
        fflush(debugfilep);
	return (jint) vdce_get_my_id();
}

/*
 * 
 * JavaDVM.vdcegetmyname()
 * 
 */
JNIEXPORT jstring JNICALL Java_JavaDVM_vdcegetmyname(
  	JNIEnv 		*env,
	jobject		obj) 
{
	char	*cret;
	jstring  ret;

	if (_debug)
	    fprintf(debugfilep,"JavaDVM.vdcegetmyname()\n");

        fflush(debugfilep);
        cret = (char *) calloc(1024,sizeof(char));
        cret=  (char *) vdce_get_my_name();
        
        if (_debug) 
                fprintf(debugfilep,"inputname=%s\n",cret);
        ret = (*env)->NewStringUTF(env,cret);
        free(cret);
 
        return ret;

}
/*
 * Class:    JavaDVM
 * Method:   vdcereportexectiontime
 */
JNIEXPORT jint JNICALL Java_JavaDVM_vdcereportexecutiontime (
	JNIEnv 	*env, 
        jobject obj,
	jint    jtime_type, 
        jlong   jtime_elapsed)
{
	jint result;

	if (_debug)
	   fprintf(debugfilep,"JavaDVM.vdcereportexecutiontime()\n");

        fflush(debugfilep);
	result = report_execution_time((int) jtime_type, (long) jtime_elapsed);
}

/*
 * Class:     JavaDVM
 * Method:    vdcesettaskname
 * Signature: ([Ljava/lang/String;[Ljava/lang/String;)I
JNIEXPORT jint JNICALL Java_JavaDVM_vdcesettaskname
  (JNIEnv *, jclass, jobjectArray, jobjectArray);
 *
 * Class:     JavaDVM
 * Method:    vdcesetinputs
 * Signature: ([Ljava/lang/String;[Ljava/lang/String;)I
JNIEXPORT jint JNICALL Java_JavaDVM_vdcesetinputs
  (JNIEnv *, jclass, jobjectArray, jobjectArray);
 *
 * Class:     JavaDVM
 * Method:    vdcesettaskoutputs
 * Signature: ([Ljava/lang/String;[Ljava/lang/String;)I
JNIEXPORT jint JNICALL Java_JavaDVM_vdcesettaskoutputs
  (JNIEnv *, jclass, jobjectArray, jobjectArray);
 */
